using ldap_cesi.DTOs.Inputs.Salarie;
using ldap_cesi.DTOs.Inputs.Service;
using ldap_cesi.Services.Interfaces;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace ldap_cesi.Controllers;
[ApiController]
[Route("api/salarie")]
public class SalarieController : ControllerBase
{
    private ISalarieService _salarieService;

    public SalarieController(ISalarieService salarieService)
    {
        _salarieService = salarieService;
    }
    
    /// <summary>
    /// Endpoint qui retourne tous les salariés.
    /// </summary>
    /// <returns>Une liste de salariés.</returns>
    [HttpGet]
    public async Task<IActionResult> GetAllSalaries()
    {
        var result = await _salarieService.GetAll();
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }     
     
    /// <summary>
    /// Endpoint qui retourne tous les salariés.
    /// </summary>
    /// <returns>Une liste de salariés.</returns>
    [HttpGet("/complete")]
    public async Task<IActionResult> GetAllSalariesComplet()
    {
        var result = await _salarieService.GetAllWithoutIService();
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui retourne le salarié correspondant à l'ID en paramètre.
    /// </summary>
    /// <param name="id">L'ID du salarié.</param>
    /// <returns>Le salarié correspondant à l'ID.</returns>
    [HttpGet("{id}")]
    public async Task<IActionResult> GetById(int id)
    {
        var result = await _salarieService.GetById(id);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }
    
    /// <summary>
    /// Endpoint qui retourne le salarié correspondant à l'ID en paramètre. Avec le nom de service et de site auxquels il apaprtient.
    /// </summary>
    /// <param name="id">L'ID du salarié.</param>
    /// <returns>Le salarié correspondant à l'ID.</returns>
    [HttpGet("/complet/{id}")]
    public async Task<IActionResult> GetSalarieCompletById(int id)
    {
        var result = await _salarieService.GetCompletById(id);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }    
    
    /// <summary>
    /// Endpoint qui retourne tout les salariés et les relations qu'il détient
    /// </summary>
    /// <returns>Tous les salariés avec leurs relations</returns>
    [HttpGet("all")]
    public async Task<IActionResult> GetAllSariesWithRelations()
    {
        var result = await _salarieService.GetAllWithRelationsAsync(s => s.IdServiceNavigation, s => s.IdSiteNavigation);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui crée un salarié.
    /// </summary>
    /// <param name="salarieInput">Les informations du salarié à créer.</param>
    /// <returns>Le salarié créé.</returns>
    [HttpPost]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> CreateSalarie([FromBody] SalarieCreateDto salarieInput)
    {
        var result = await _salarieService.Create(salarieInput);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui met à jour un salarié.
    /// </summary>
    /// <param name="salarieInput">Les informations du salarié à mettre à jour.</param>
    /// <returns>Le salarié mis à jour.</returns>
    [HttpPut]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> UpdateSalarie([FromBody] SalarieUpdateDto salarieInput)
    {
        var result = await _salarieService.Update(salarieInput);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui supprime un salarié.
    /// </summary>
    /// <param name="id">L'ID du salarié à supprimer.</param>
    /// <returns>Un message de confirmation de suppression.</returns>
    [HttpDelete("{id}")]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> DeleteSalarie(int id)
    {
        var result = await _salarieService.Delete(id);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }
    
    /// <summary>
    /// Récupère les salariés appartenant à un site spécifique
    /// </summary>
    /// <param name="siteId">Identifiant du site</param>
    /// <returns>Liste des salariés du site</returns>
    [HttpGet("site/{siteId}")]
    public async Task<IActionResult> GetSalariesBySite(int siteId)
    {
        var result = await _salarieService.GetSalariesBySite(siteId);
        return result.Success ? Ok(result.Data) : NotFound(result.Message);
    }
    
    /// <summary>
    /// Récupère les salariés appartenant à un service spécifique
    /// </summary>
    /// <param name="serviceId">Identifiant du service</param>
    /// <returns>Liste des salariés du service</returns>
    [HttpGet("service/{serviceId}")]
    public async Task<IActionResult> GetSalariesByService(int serviceId)
    {
        var result = await _salarieService.GetSalariesByService(serviceId);
        return result.Success ? Ok(result.Data) : NotFound(result.Message);
    }

    /// <summary>
    /// Endpoint qui met à jour partiellement un salarié.
    /// </summary>
    /// <param name="id">L'ID du salarié.</param>
    /// <param name="salariePatch">Les informations du salarié à mettre à jour partiellement.</param>
    /// <returns>Le salarié mis à jour partiellement.</returns>
    // [HttpPatch("{id}")]
    // [Authorize(Roles = "admin")]
    // public async Task<IActionResult> PatchSalarie(int id, [FromBody] SalariePatchDto salariePatch)
    // {
    //     var result = await _salarieService.Patch(id, salariePatch);
    //     return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    // }
}
