using ldap_cesi.DTOs.Inputs.Service;
using ldap_cesi.Services.Interfaces;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace ldap_cesi.Controllers;

[ApiController]
[Route("api/service")]
public class ServicesController : ControllerBase
{
    private IServiceService _serviceService;

    public ServicesController(IServiceService serviceService)
    {
        _serviceService = serviceService;
    }
    
    /// <summary>
    /// Endpoint qui retourne tous les services.
    /// </summary>
    /// <returns>Retourne tous les services.</returns>
    [HttpGet]
    public async Task<IActionResult> GetServices()
    {
        var result = await _serviceService.GetAll();
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui retourne le service correspondant à l'id en paramètre.
    /// </summary>
    /// <param name="id">L'ID du service.</param>
    /// <returns>Le service correspondant à l'ID.</returns>
    [HttpGet("{id}")]
    public async Task<IActionResult> GetServiceById(int id)
    {
        var result = await _serviceService.GetById(id);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui crée un service.
    /// </summary>
    /// <param name="serviceInputDto">Les informations du service à créer.</param>
    /// <returns>Le service créé.</returns>
    [HttpPost]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> CreateService([FromBody] ServiceCreateDto serviceInputDto)
    {
        var result = await _serviceService.Create(serviceInputDto);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui met à jour un service.
    /// </summary>
    /// <param name="serviceUpdateDto">Les informations du service à mettre à jour.</param>
    /// <returns>Le service mis à jour.</returns>
    [HttpPut]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> UpdateService([FromBody] ServiceUpdateDto serviceUpdateDto)
    {
        var result = await _serviceService.Update(serviceUpdateDto);
        return result.Success ? Ok(result.Data) : BadRequest(result.Message);
    }

    /// <summary>
    /// Endpoint qui supprime un service.
    /// </summary>
    /// <param name="id">L'ID du service à supprimer.</param>
    /// <returns>Un message de confirmation de suppression.</returns>
    [HttpDelete("{id}")]
    [Authorize(Roles = "admin")]
    public async Task<IActionResult> DeleteService(int id)
    {
        var result = await _serviceService.Delete(id);
        return result.Success ? Ok(result.Message) : BadRequest(result.Message);
    }
}