using AutoMapper;
using ldap_cesi.DTOs.Inputs.Site;
using ldap_cesi.Entities;
using ldap_cesi.Models;
using ldap_cesi.Repository.Services;
using ldap_cesi.Services.Interfaces;
using ldap_cesi.Validator.Site;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Threading.Tasks;
using FluentValidation;

namespace ldap_cesi.Services;

public class ServiceBase<T, TDto, TCreateDto, TUpdateDto> : IServiceBase<T, TDto, TCreateDto, TUpdateDto>
    where T : class
    where TDto : class
    where TCreateDto : class
    where TUpdateDto : class
{
    protected readonly IRepositoryBase<T> _repository;
    protected readonly IMapper _mapper;
    protected readonly ILogger<ServiceBase<T, TDto, TCreateDto, TUpdateDto>> _logger;
    protected readonly IValidator<TCreateDto> _createDtoValidator;
    protected readonly IValidator<TUpdateDto> _updateDtoValidator;

    public ServiceBase(IRepositoryBase<T> repository, IMapper mapper, ILogger<ServiceBase<T, TDto, TCreateDto, TUpdateDto>> logger, IValidator<TCreateDto> createDtoValidator, IValidator<TUpdateDto> updateDtoValidator)
    {
        _repository = repository;
        _mapper = mapper;
        _logger = logger;
        _createDtoValidator = createDtoValidator;
        _updateDtoValidator = updateDtoValidator;
    }

    public virtual async Task<IResponseDataModel<List<T>>> GetAll()
    {
        try
        {
            var entities = await _repository.GetAllAsync();
            return new ResponseDataModel<List<T>>
            {
                Success = true,
                Data = entities,
                StatusCode = 200,
                Message = "Liste des entités récupérée avec succès."
            };
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Une erreur s'est produite lors de la récupération des entités.");
            return new ResponseDataModel<List<T>>
            {
                Success = false,
                Message = "Une erreur s'est produite lors de la récupération des entités.",
                StatusCode = 500
            };
        }
    }

    public virtual async Task<IResponseDataModel<T>> GetById(int id)
    {
        try
        {
            var entity = await _repository.GetByIdAsync(id);
            if (entity == null)
            {
                return new ResponseDataModel<T>
                {
                    Success = false,
                    Message = $"Aucune entité trouvée avec l'identifiant {id}.",
                    StatusCode = 404
                };
            }

            return new ResponseDataModel<T>
            {
                Success = true,
                Data = entity,
                StatusCode = 200,
                Message = "Entité récupérée avec succès."
            };
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, $"Une erreur s'est produite lors de la récupération de l'entité avec l'identifiant {id}.");
            return new ResponseDataModel<T>
            {
                Success = false,
                Message = "Une erreur s'est produite lors de la récupération de l'entité.",
                StatusCode = 500
            };
        }
    }
    
    public virtual async Task<IResponseDataModel<List<TDto>>> GetAllWithRelationsAsync(params Expression<Func<T, object>>[] relationsAInclure)
    {
        try
        {
            var entities = await _repository.GetAllWithRelationsAsync(relationsAInclure);
            var dtos = _mapper.Map<List<TDto>>(entities);
            return new ResponseDataModel<List<TDto>>
            {
                Success = true,
                Data = dtos,
                StatusCode = 200,
                Message = "Liste des entités récupérée avec succès."
            };
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Une erreur s'est produite lors de la récupération des entités.");
            return new ResponseDataModel<List<TDto>>
            {
                Success = false,
                Message = "Une erreur s'est produite lors de la récupération des entités.",
                StatusCode = 500
            };
        }
    }
    
    public virtual async Task<IResponseDataModel<TDto>> GetByIdWithRelations(int id, params Expression<Func<T, object>>[] relationsAInclure)
    {
        try
        {
            var entity = await _repository.GetWithRelationsAsync(id, relationsAInclure);
            if (entity == null)
            {
                return new ResponseDataModel<TDto>
                {
                    Success = false,
                    Message = $"Aucune entité trouvée avec l'identifiant {id}.",
                    StatusCode = 404
                };
            }

            var dto = _mapper.Map<TDto>(entity);
            return new ResponseDataModel<TDto>
            {
                Success = true,
                Data = dto,
                StatusCode = 200,
                Message = "Entité avec relations récupérée avec succès."
            };
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, $"Une erreur s'est produite lors de la récupération de l'entité avec l'identifiant {id}.");
            return new ResponseDataModel<TDto>
            {
                Success = false,
                Message = "Une erreur s'est produite lors de la récupération de l'entité avec relations.",
                StatusCode = 500
            };
        }
    }

    public virtual async Task<IResponseDataModel<T>> Create(TCreateDto dto)
    {
        try
        {
            var validationResult = _createDtoValidator.Validate(dto);
            if (!validationResult.IsValid)
            {
                return new ResponseDataModel<T>
                {
                    Success = false,
                    Message = string.Join("; ", validationResult.Errors.Select(e => e.ErrorMessage)),
                    StatusCode = 400
                };
            }
            var entity = _mapper.Map<T>(dto);
            var createdEntity = await _repository.AddAsync(entity);
            return new ResponseDataModel<T>
            {
                Success = createdEntity != null,
                Data = createdEntity,
                StatusCode = createdEntity != null ? 201 : 500,
                Message = createdEntity != null ? "Entité créée avec succès." : "Échec de la création de l'entité."
            };
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Une erreur s'est produite lors de la création de l'entité.");
            return new ResponseDataModel<T>
            {
                Success = false,
                Message = "Une erreur s'est produite lors de la création de l'entité.",
                StatusCode = 500
            };
        }
    }

    public virtual async Task<IResponseDataModel<T>> Update(TUpdateDto dto)
    {
        try
        {
            var validationResult = _updateDtoValidator.Validate(dto);
            if (!validationResult.IsValid)
            {
                return new ResponseDataModel<T>
                {
                    Success = false,
                    Message = string.Join("; ", validationResult.Errors.Select(e => e.ErrorMessage)),
                    StatusCode = 400
                };
            }
            var entity = _mapper.Map<T>(dto);
            bool isUpdated = await _repository.UpdateAsync(entity);
            return new ResponseDataModel<T>
            {
                Success = isUpdated,
                Data = isUpdated ? entity : default,
                StatusCode = isUpdated ? 200 : 500,
                Message = isUpdated ? "Entité mise à jour avec succès." : "Échec de la mise à jour de l'entité."
            };
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Une erreur s'est produite lors de la mise à jour de l'entité.");
            return new ResponseDataModel<T>
            {
                Success = false,
                Message = "Une erreur s'est produite lors de la mise à jour de l'entité.",
                StatusCode = 500
            };
        }
    }

    public virtual async Task<IResponseDataModel<string>> Delete(int id)
    {
        try
        {
            var entity = await _repository.GetByIdAsync(id);
            if (entity == null)
            {
                return new ResponseDataModel<string>
                {
                    Success = false,
                    Message = $"Aucune entité trouvée avec l'identifiant {id}.",
                    StatusCode = 404
                };
            }

            var isDeleted = await _repository.DeleteAsync(entity);
            return new ResponseDataModel<string>
            {
                Success = isDeleted,
                Data = isDeleted ? id.ToString() : null,
                StatusCode = isDeleted ? 200 : 500,
                Message = isDeleted ? "Entité supprimée avec succès." : "Échec de la suppression de l'entité."
            };
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, $"Une erreur s'est produite lors de la suppression de l'entité avec l'identifiant {id}.");
            return new ResponseDataModel<string>
            {
                Success = false,
                Message = "Une erreur s'est produite lors de la suppression de l'entité.",
                StatusCode = 500
            };
        }
    }
}